==> harmonize-3.0.0-rc1.tar.gz: harmonize/src/EventSubscriber/HarmonyEventSubscriber.php <==
<?php

namespace Drupal\harmonize\EventSubscriber;

use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\harmonize\Constants\HarmonizeConfig;
use Drupal\harmonize\Constants\HarmonizerFlags;
use Drupal\harmonize\Event\HarmonizationEvent;
use Drupal\harmonize\Constants\HarmonizationEvents;
use Drupal\harmonize\Harmonizer\HarmonizerInterface;
use Drupal\harmonize\Service\Harmonics;
use Drupal\harmonize\Service\Harmonize;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Subscribes to events to apply modifications of harmonization data.
 *
 * @package Drupal\custom_events\EventSubscriber
 *
 * @noinspection PhpUnused
 */
class HarmonyEventSubscriber implements EventSubscriberInterface {

  /**
   * Define the path that will be crawled in the theme for Handshake files.
   *
   * This is the path relative to the root of the Drupal theme.
   *
   * @var string
   */
  public const ROOT = '/harmony';

  /**
   * Define the path that will be crawled in the theme for Handshake files.
   *
   * This is the path relative to the root of the Drupal theme.
   *
   * @var string
   */
  public const FILE_EXTENSION = '.harmony.php';

  /**
   * The Harmonics service injected through DI.
   *
   * @var \Drupal\harmonize\Service\Harmonics
   */
  private Harmonics $harmonics;

  /**
   * The Theme Manager service injected through DI.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  private ThemeManagerInterface $themeManager;


  /**
   * The Theme Extension List service injected through DI.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  private ExtensionList $themeExtensionList;

  /**
   * A flag that determines whether we want to stop including files.
   */
  private bool $break = FALSE;

  /**
   * A flag that determines whether we found a file.
   */
  private bool $loadedFile = FALSE;

  /**
   * HarmonyEventSubscriber constructor.
   *
   * @param \Drupal\harmonize\Service\Harmonics $harmonics
   *   The Harmonics service injected through DI.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   Theme Manager injected through DI.
   * @param \Drupal\Core\Extension\ExtensionList $themeExtensionList
   *   Drupal's Theme Extension List service.
   */
  public function __construct(
    Harmonics $harmonics,
    ThemeManagerInterface $themeManager,
    ExtensionList $themeExtensionList
  ) {
    $this->harmonics = $harmonics;
    $this->themeManager = $themeManager;
    $this->themeExtensionList = $themeExtensionList;
  }

  /**
   * {@inheritdoc}
   *
   * @noinspection PhpArrayShapeAttributeCanBeAddedInspection
   */
  public static function getSubscribedEvents() : array {
    return [
      HarmonizationEvents::CORE => ['onHarmonize', 100],
    ];
  }

  /**
   * React to a Harmonizer event.
   *
   * @param \Drupal\harmonize\Event\HarmonizationEvent $event
   *   Event that occurred, containing the harmonizer that fired the event.
   *
   * @noinspection PhpUnused
   */
  public function onHarmonize(HarmonizationEvent $event) : void {
    // If Harmony File Discovery is disabled, we can stop here.
    if (!Harmonize::$config[HarmonizeConfig::MAIN__HARMONY_FILE_DISCOVERY]) {
      $event->harmonizer->trace(t("[Harmony Event] Harmony file discovery is disabled in module configurations...Continuing"));
      return;
    }
    else {
      $event->harmonizer->trace(t("[Harmony Event] Harmony file discovery is enabled. Proceeding"));
    }

    $harmonics = $this->harmonics;
    $harmony = [];
    $consignment = $event->harmonizer->getData();

    // The default theme for this is the active one.
    $theme = $this->themeManager->getActiveTheme()->getName();

    // If the THEME flag is set, we want to change the directory.
    // Note that the site WILL crash if an invalid theme is given.
    // We want this to happen. :)
    if ($event->harmonizer->hasFlag(HarmonizerFlags::THEME) && !empty($event->harmonizer->getFlag(HarmonizerFlags::THEME))) {
      $theme = $event->harmonizer->getFlag(HarmonizerFlags::THEME);
    }

    // Trace.
    $event->harmonizer->trace(t("[Harmony Event] Theme has been resolved as <strong><code>[@theme]</code></strong>", ['@theme' => $theme]));

    // Set our directory.
    $directory = $this->themeExtensionList->getPath($theme) . self::ROOT;
    if (file_exists($directory . '/functions.php')) {
      /* @noinspection PhpIncludeInspection */
      include_once $directory . '/functions.php';
    }

    // Get signature parts.
    /* @see \Drupal\harmonize\Harmonizer\Harmonizer::getSignatureParts() */
    $signatureParts = $event->harmonizer->getSignatureParts(TRUE);

    // If 'FCFS' is set, we only want to get the first harmony file we find.
    if ($this->isFcfs()) {
      $event->harmonizer->trace(t("[Harmony Event] <strong>FCFS</strong> Algorithm Detected...Only a single harmony file will be loaded"));
      // Okay, cool stuff now.
      // We reverse the signature parts array before proceeding.
      // This is because we are only going to be trying to get a single harmony
      // file.
      $signatureParts = array_reverse($signatureParts);
    }

    // With each part of the signature, include files matching the part.
    foreach ($signatureParts as $part) {
      // Include all files we can find.
      $this->includeAll(
        $directory, $part, $event->harmonizer, $consignment, $harmony, $harmonics
      );

      // If this is set to break, we can stop.
      if ($this->break) {
        break;
      }
    }

    // If no files were loaded, we can stop here.
    if (!$this->loadedFile) {
      $event->harmonizer->trace(t("[Harmony Event] No harmony files were found. Data will be returned in its current state"), TRUE);
    }
  }

  /**
   * Scan the path, recursively including all PHP files.
   *
   * @param string $dir
   *   Directory to look through to include files.
   * @param string $filename
   *   Filename to look for recursively.
   * @param \Drupal\harmonize\Harmonizer\HarmonizerInterface $harmonizer
   *   Data from the harmonizer, already processed by default.
   * @param mixed $consignment
   *   Data from the harmonizer, already processed by default.
   * @param mixed $harmony
   *   Data of the harmonizer that will be released to templates.
   * @param \Drupal\harmonize\Service\Harmonics $harmonics
   *   The Harmonics service passed down.
   */
  private function includeAll(
    string $dir,
    string $filename,
    HarmonizerInterface $harmonizer,
    mixed &$consignment,
    mixed &$harmony,
    Harmonics &$harmonics
  ) : void {
    if ($this->break) {
      return;
    }
    $escaped_filename = str_replace('.', '\.', $filename);
    $escaped_file_extension = str_replace(
      '.', '\.', self::FILE_EXTENSION
    );

    $scan = glob("$dir/*");

    foreach ($scan as $path) {
      $test = '/\/' . $escaped_filename . $escaped_file_extension . '$/';
      if (preg_match($test, $path)) {
        // Include the file.
        include $path;

        // Confirm that we loaded at least one file.
        $this->loadedFile = TRUE;

        // Trace.
        $harmonizer->trace(t("[Harmony Event] Harmony file found: <strong><code>@file</code></strong> has been loaded and data alterations have been applied", ['@file' => $path]));

        // If the harmony object is empty, that means it wasn't reformatted.
        // We'll return the original consignment.
        if (!empty($harmony)) {
          $harmonizer->setData($harmony);
          $harmonizer->trace(t("[Harmony Event] <strong><code>\$harmony</code></strong> variable was detected...Data has been overwritten with the contents of the variable"), TRUE);
        }
        else {
          $harmonizer->setData($consignment);
          $harmonizer->trace(t("[Harmony Event] Data was modified and returned with alterations from files"), TRUE);
        }

        // If we're in FCFS mode, we want to stop after one include.
        if ($this->isFcfs()) {
          $harmonizer->trace(t("[Harmony Event] Due to <strong>FCFS</strong>, no other harmony files will be loaded. Proceeding", [
            '@file' => $path,
          ]));
          $this->break = TRUE;
        }
      }
      elseif (is_dir($path)) {
        $this->includeAll($path, $filename, $harmonizer, $consignment, $harmony, $harmonics);
      }
    }
  }

  /**
   * Helper function to tell us if we're in 'FCFS' mode.
   *
   * @return bool
   *   Returns TRUE if we're in FCFS mode, returns FALSE otherwise.
   */
  private function isFcfs() : bool {
    return Harmonize::$config[HarmonizeConfig::MAIN__LOADING_ALGORITHM] === 'fcfs';
  }

}

==> harmonize-3.0.1.tar.gz: harmonize/src/EventSubscriber/HarmonyEventSubscriber.php <==
<?php

namespace Drupal\harmonize\EventSubscriber;

use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\harmonize\Constants\HarmonizeConfig;
use Drupal\harmonize\Constants\HarmonizerFlags;
use Drupal\harmonize\Event\HarmonizationEvent;
use Drupal\harmonize\Constants\HarmonizationEvents;
use Drupal\harmonize\Harmonizer\HarmonizerInterface;
use Drupal\harmonize\Service\Harmonics;
use Drupal\harmonize\Service\Harmonize;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Subscribes to events to apply modifications of harmonization data.
 *
 * @package Drupal\custom_events\EventSubscriber
 *
 * @noinspection PhpUnused
 */
class HarmonyEventSubscriber implements EventSubscriberInterface {
  use StringTranslationTrait;

  /**
   * Define the path that will be crawled in the theme for Handshake files.
   *
   * This is the path relative to the root of the Drupal theme.
   *
   * @var string
   */
  public const ROOT = '/harmony';

  /**
   * Define the path that will be crawled in the theme for Handshake files.
   *
   * This is the path relative to the root of the Drupal theme.
   *
   * @var string
   */
  public const FILE_EXTENSION = '.harmony.php';

  /**
   * The Harmonics service injected through DI.
   *
   * @var \Drupal\harmonize\Service\Harmonics
   */
  private Harmonics $harmonics;

  /**
   * The Theme Manager service injected through DI.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  private ThemeManagerInterface $themeManager;


  /**
   * The Theme Extension List service injected through DI.
   *
   * @var \Drupal\Core\Extension\ExtensionList
   */
  private ExtensionList $themeExtensionList;

  /**
   * A flag that determines whether we want to stop including files.
   *
   * @var bool
   */
  private bool $break = FALSE;

  /**
   * A flag that determines whether we found a file.
   *
   * @var bool
   */
  private bool $loadedFile = FALSE;

  /**
   * HarmonyEventSubscriber constructor.
   *
   * @param \Drupal\harmonize\Service\Harmonics $harmonics
   *   The Harmonics service injected through DI.
   * @param \Drupal\Core\Theme\ThemeManagerInterface $themeManager
   *   Theme Manager injected through DI.
   * @param \Drupal\Core\Extension\ExtensionList $themeExtensionList
   *   Drupal's Theme Extension List service.
   */
  public function __construct(
    Harmonics $harmonics,
    ThemeManagerInterface $themeManager,
    ExtensionList $themeExtensionList
  ) {
    $this->harmonics = $harmonics;
    $this->themeManager = $themeManager;
    $this->themeExtensionList = $themeExtensionList;
  }

  /**
   * {@inheritdoc}
   *
   * @noinspection PhpArrayShapeAttributeCanBeAddedInspection
   */
  public static function getSubscribedEvents() : array {
    return [
      HarmonizationEvents::CORE => ['onHarmonize', 100],
    ];
  }

  /**
   * React to a Harmonizer event.
   *
   * @param \Drupal\harmonize\Event\HarmonizationEvent $event
   *   Event that occurred, containing the harmonizer that fired the event.
   *
   * @noinspection PhpUnused
   */
  public function onHarmonize(HarmonizationEvent $event) : void {
    // Reset our trackers.
    $this->break = FALSE;
    $this->loadedFile = FALSE;

    // If Harmony File Discovery is disabled, we can stop here.
    if (!Harmonize::$config[HarmonizeConfig::MAIN__HARMONY_FILE_DISCOVERY]) {
      $event->harmonizer->trace($this->t("[Harmony Event] Harmony file discovery is disabled in module configurations...Continuing"));
      return;
    }
    else {
      $event->harmonizer->trace($this->t("[Harmony Event] Harmony file discovery is enabled. Proceeding"));
    }

    $harmonics = $this->harmonics;
    $harmony = [];
    $consignment = $event->harmonizer->getData();

    // The default theme for this is the active one.
    $theme = $this->themeManager->getActiveTheme()->getName();

    // If the THEME flag is set, we want to change the directory.
    // Note that the site WILL crash if an invalid theme is given.
    // We want this to happen. :)
    if ($event->harmonizer->hasFlag(HarmonizerFlags::THEME) && !empty($event->harmonizer->getFlag(HarmonizerFlags::THEME))) {
      $theme = $event->harmonizer->getFlag(HarmonizerFlags::THEME);
    }

    // Trace.
    $event->harmonizer->trace($this->t("[Harmony Event] Theme has been resolved as <strong><code>[@theme]</code></strong>", ['@theme' => $theme]));

    // Set our directory.
    $directory = $this->themeExtensionList->getPath($theme) . self::ROOT;
    if (file_exists($directory . '/functions.php')) {
      /* @noinspection PhpIncludeInspection */
      include_once $directory . '/functions.php';
    }

    // Get signature parts.
    /* @see \Drupal\harmonize\Harmonizer\Harmonizer::getSignatureParts() */
    $signatureParts = $event->harmonizer->getSignatureParts(TRUE);

    // If 'FCFS' is set, we only want to get the first harmony file we find.
    if ($this->isFcfs()) {
      $event->harmonizer->trace($this->t("[Harmony Event] <strong>FCFS</strong> Algorithm Detected...Only a single harmony file will be loaded"));
      // Okay, cool stuff now.
      // We reverse the signature parts array before proceeding.
      // This is because we are only going to be trying to get a single harmony
      // file.
      $signatureParts = array_reverse($signatureParts);
    }

    // With each part of the signature, include files matching the part.
    foreach ($signatureParts as $part) {
      // Include all files we can find.
      $this->includeAll(
        $directory, $part, $event->harmonizer, $consignment, $harmony, $harmonics
      );

      // If this is set to break, we can stop.
      if ($this->break) {
        break;
      }
    }

    // If no files were loaded, we can stop here.
    if (!$this->loadedFile) {
      $event->harmonizer->trace($this->t("[Harmony Event] No harmony files were found. Data will be returned in its current state"), TRUE);
    }
  }

  /**
   * Scan the path, recursively including all PHP files.
   *
   * @param string $dir
   *   Directory to look through to include files.
   * @param string $filename
   *   Filename to look for recursively.
   * @param \Drupal\harmonize\Harmonizer\HarmonizerInterface $harmonizer
   *   Data from the harmonizer, already processed by default.
   * @param mixed $consignment
   *   Data from the harmonizer, already processed by default.
   * @param mixed $harmony
   *   Data of the harmonizer that will be released to templates.
   * @param \Drupal\harmonize\Service\Harmonics $harmonics
   *   The Harmonics service passed down.
   *
   * @noinspection PhpMissingParamTypeInspection
   */
  private function includeAll(
    string $dir,
    string $filename,
    HarmonizerInterface $harmonizer,
    &$consignment,
    &$harmony,
    Harmonics &$harmonics
  ) : void {
    if ($this->break) {
      return;
    }
    $escaped_filename = str_replace('.', '\.', $filename);
    $escaped_file_extension = str_replace(
      '.', '\.', self::FILE_EXTENSION
    );

    $scan = glob("$dir/*");

    foreach ($scan as $path) {
      $test = '/\/' . $escaped_filename . $escaped_file_extension . '$/';
      if (preg_match($test, $path)) {
        // Include the file.
        include $path;

        // Confirm that we loaded at least one file.
        $this->loadedFile = TRUE;

        // Trace.
        $harmonizer->trace($this->t("[Harmony Event] Harmony file found: <strong><code>@file</code></strong> has been loaded and data alterations have been applied", ['@file' => $path]));

        // If the harmony object is empty, that means it wasn't reformatted.
        // We'll return the original consignment.
        if (!empty($harmony)) {
          $harmonizer->setData($harmony);
          $harmonizer->trace($this->t("[Harmony Event] <strong><code>\$harmony</code></strong> variable was detected...Data has been overwritten with the contents of the variable"), TRUE);
        }
        else {
          $harmonizer->setData($consignment);
          $harmonizer->trace($this->t("[Harmony Event] Data was modified and returned with alterations from files"), TRUE);
        }

        // If we're in FCFS mode, we want to stop after one include.
        if ($this->isFcfs()) {
          $harmonizer->trace($this->t("[Harmony Event] Due to <strong>FCFS</strong>, no other harmony files will be loaded. Proceeding", [
            '@file' => $path,
          ]));
          $this->break = TRUE;
        }
      }
      elseif (is_dir($path)) {
        $this->includeAll($path, $filename, $harmonizer, $consignment, $harmony, $harmonics);
      }
    }
  }

  /**
   * Helper function to tell us if we're in 'FCFS' mode.
   *
   * @return bool
   *   Returns TRUE if we're in FCFS mode, returns FALSE otherwise.
   */
  private function isFcfs() : bool {
    return Harmonize::$config[HarmonizeConfig::MAIN__LOADING_ALGORITHM] === 'fcfs';
  }

}
